import pytest


class TestClass:
    def test_capturing_stdout(self):
        from OTils.capturing import Capturing
        message = "This is going to be captured"
        with Capturing() as capture:
            print(message)

        captured_message = capture["out"][0]
        assert captured_message == message

    def test_capturing_stderr(self):
        import sys
        from OTils.capturing import Capturing
        message = "This is going to be captured"
        with Capturing(capture_stderr=True) as capture:
            print(message, file=sys.stderr)

        captured_message = capture["err"][0]
        assert captured_message == message


class TestSizeUtils:
    def test_file_size(self):
        from OTils.size_utils import file_size
        script_size = file_size(__file__)
        assert script_size > 0

    def test_file_list_size(self):
        from OTils.size_utils import file_list_size
        files = [__file__, __file__]
        size = file_list_size(files)
        assert size > 0

    def test_convert_size(self):
        from OTils.size_utils import convert_size
        pairs = [("1.0KB", 1024),
                 ("1.0MB", 1024 ** 2),
                 ("1.0GB", 1024 ** 3),
                 ("-1.0GB", -1024 ** 3),
                 ("0B", 0),
                 ]

        for string, value in pairs:
            assert string == convert_size(value)

    def test_parse_size(self):
        from OTils.size_utils import parse_size
        assert 1024 == parse_size("1.0KB")
        with pytest.raises(AssertionError):
            parse_size("patata")


class TestFileDealer:
    def test_file_dealer_number_of_files(self):
        from OTils.file_dealer import file_dealer
        files = file_dealer(number_of_files=2)
        assert len(files) == 2

    def test_file_dealer_total_size(self):
        from OTils.file_dealer import file_dealer
        files = file_dealer(total_size="500MB")
        assert len(files) > 0

    def test_file_dealer_no_arg(self):
        from OTils.file_dealer import file_dealer
        with pytest.raises(AssertionError):
            file_dealer()

    def test_file_dealer_folder_test(self):
        from OTils.configuration import configuration
        # Ugly thing to do
        valid_path = configuration.folders.data
        configuration.folders.data = "None"

        from OTils.file_dealer import file_dealer
        with pytest.raises(IOError):
            file_dealer(number_of_files=1)
        # Restore proper path
        configuration.folders.data = valid_path

    def test_give_me_a_dataset(self):
        from OTils.configuration import configuration
        assert configuration.folders.data != "None"
        from OTils.file_dealer import give_me_a_dataset
        import xarray
        ds = give_me_a_dataset()
        assert isinstance(ds, xarray.Dataset)

    def test_give_me_data(self):
        from OTils.file_dealer import give_me_data
        import numpy
        data = give_me_data()
        assert isinstance(data, numpy.ndarray)


class TestMemoryTracker:
    def test_class(self):
        from OTils.memory_tracker import MemoryTracker
        """
        Some example code using the class MemoryTracker
        """
        import time
        experiment_duration = 2  # seconds

        with MemoryTracker() as tracker:
            # Do something for a couple of seconds
            start_time = time.perf_counter()

            # To have something to observe, lets initialize a list and fill it up in each iteration
            to_fill = []
            # Start a second tracker with a Context Manager
            while time.perf_counter() - start_time < experiment_duration:
                # Add some stuff to the list
                to_fill.append([x for x in range(100000)])
                # Print status:
                print(tracker)

                # Sleep
                time.sleep(0.5)
            pass


class TestSuggester:
    def test_suggester(self):
        from OTils.suggester import suggester
        # List of possible commands inspired by autosubmit
        possible_commands = ["run", "create", "recovery", "monitor", "delete"]

        command = "creat"
        suggestion = suggester(
            options=possible_commands,
            prior=command,
                               )
        assert suggestion == "create"

    def test_bad_prior(self):
        from OTils.suggester import suggester
        # List of possible commands inspired by autosubmit
        possible_commands = ["run", "create", "recovery", "monitor", "delete"]

        command = "x"
        suggestion = suggester(
            options=possible_commands,
            prior=command,
        )
        assert suggestion is None


class TestCommandLineInterfaceAid:
    def test_cli(self):
        """
        The test is a bit long.
        But can be used as an example too.
        Returns:

        """

        from OTils.command_line_interface_aid import CommandLineInterface

        class DummyClass:
            def __init__(self):
                self.cli = CommandLineInterface()
                self.set_up_cli()

            def set_up_cli(self):
                self.cli.add_action(name="list",
                                    aliases=["--list", "-l"],
                                    action=self.dummy,
                                    )

                self.cli.add_action(name="integer",
                                    aliases=["--integer", "-i"],
                                    action=self.dummy_retrieve,
                                    help="Integer action.",
                                    type=int,
                                    )
                self.cli.add_action(name="float",
                                    aliases=["--float", "-f"],
                                    action=self.dummy_retrieve,
                                    help="Float action.",
                                    type=float,
                                    )

            def dummy(self, *args, **kwargs):
                print(f"Dummy called from {self}")
                print("Yummy dummy!")

            @staticmethod
            def dummy_retrieve(dummy_id):
                print(f"Received {type(dummy_id)}: {dummy_id}")

            def run(self, args=None):
                return self.cli.parse(args)

        dummy = DummyClass()

        cases = [
            ["--integer", "1"],
            ["--float", "1.2"],
            ["-l"],
        ]
        for case in cases:
            dummy.run(case)
